//
//  AppDelegate.swift
//  TemperatureSensor
//
//  Created by Phidgets on 2018-02-16.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import Cocoa
import Phidget22Swift

@NSApplicationMain
class AppDelegate: NSObject, NSApplicationDelegate {

    @IBOutlet weak var window: NSWindow!
    let ch = TemperatureSensor()
    @IBOutlet weak var phidgetInfoBox: PhidgetInfoBox!
    
    //GUI
    @IBOutlet weak var changeTriggerSlider: NSSlider!
    @IBOutlet weak var minChangeTriggerLabel: NSTextField!
    @IBOutlet weak var maxChangeTriggerLabel: NSTextField!
    @IBOutlet weak var changeTriggerLabel: NSTextField!
    @IBOutlet weak var dataIntervalSlider: NSSlider!
    @IBOutlet weak var minDataIntervalLabel: NSTextField!
    @IBOutlet weak var maxDataIntervalLabel: NSTextField!
    @IBOutlet weak var dataIntervalLabel: NSTextField!
    @IBOutlet weak var dataIntervalBox: NSBox!
    @IBOutlet weak var changeTriggerBox: NSBox!
    @IBOutlet weak var thermocoupleBox: NSBox!
    @IBOutlet weak var thermocoupleTypePopup: NSPopUpButton!
    @IBOutlet weak var rtdBox: NSBox!
    @IBOutlet weak var rtdTypePopup: NSPopUpButton!
    @IBOutlet weak var rtdWireSetupPopup: NSPopUpButton!
    @IBOutlet weak var dataBox: NSBox!
    @IBOutlet weak var temperatureLabel: NSTextField!
    
    //Error Window
    @IBOutlet weak var errorEventWindow: NSPanel!
    @IBOutlet var errorEventLog: NSTextView!
    @IBOutlet weak var errorEventCount: NSTextField!
    var errorCounter = 0


    //MARK: View events
    func applicationDidFinishLaunching(_ aNotification: Notification) {
        do {
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
            
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.temperatureChange.addHandler(temperaturechange_handler)
            
            /*
             * Please review the Phidget22 channel matching documentation for details on the device
             * and class architecture of Phidget22, and how channels are matched to device features.
             */
            
            /*
             * Specifies the serial number of the device to attach to.
             * For VINT devices, this is the hub serial number.
             *
             * The default is any device.
             */
            // try ch.setDeviceSerialNumber(<YOUR DEVICE SERIAL NUMBER>)
            
            /*
             * For VINT devices, this specifies the port the VINT device must be plugged into.
             *
             * The default is any port.
             */
            // try ch.setHubPort(0)
            
            /*
             * Specifies which channel to attach to.  It is important that the channel of
             * the device is the same class as the channel that is being opened.
             *
             * The default is any channel.
             */
            // try ch.setChannel(0)
            
            /*
             * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
             * In a normal environment this can be done automatically by enabling server discovery, which
             * will cause the client to discovery and connect to available servers.
             *
             * To force the channel to only match a network Phidget, set remote to 1.
             */
            
            // try Net.enableServerDiscovery(serverType: .deviceRemote)
            // try ch.setIsRemote(true)
            
            /*
             openCmdLine allows command line arguments to be passed to this program in order to
             configure channel matching.
             If you have modified the code and added your own channel matching, comment out
             the openCmdLine call, and simply replace it with the following line:
             */
            //try ch.open()
            
            try phidgetInfoBox.openCmdLine(phid: ch as Phidget)
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    func applicationShouldTerminateAfterLastWindowClosed(_ sender: NSApplication) -> Bool {
        return true
    }
    
    func applicationWillTerminate(_ aNotification: Notification) {
        //Safely close example
        do{
            print("Closing")
            ch.attach.removeAllHandlers()
            ch.detach.removeAllHandlers()
            ch.error.removeAllHandlers()
            ch.temperatureChange.removeAllHandlers()
            try ch.close()
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event handlers
    func attach_handler(sender: Phidget){
        do{
            let attachedDevice = sender as! TemperatureSensor
            let subclass = try attachedDevice.getChannelSubclass()
            
            let minChangeTrigger = try attachedDevice.getMinTemperatureChangeTrigger()
            let maxChangeTrigger = try attachedDevice.getMaxTemperatureChangeTrigger()
            let changeTrigger = try attachedDevice.getTemperatureChangeTrigger()
            let minDataInterval = try attachedDevice.getMinDataInterval()
            let maxDataInterval = try attachedDevice.getMaxDataInterval()
            let dataInterval = try attachedDevice.getDataInterval()
            var thermocoupleType:ThermocoupleType?
            var rtdType:RTDType?
            var rtdWireSetup:RTDWireSetup?
            var hideTriggerBox = false
            
            if(subclass == ChannelSubclass.temperatureSensorThermocouple){
                thermocoupleType = try ch.getThermocoupleType()
            }
            else if(subclass == ChannelSubclass.temperatureSensorRTD){
                rtdType = try ch.getRTDType()
                rtdWireSetup = try ch.getRTDWireSetup()
            }
            
            
            DispatchQueue.main.async {
                self.phidgetInfoBox.fillPhidgetInfo(device: sender)
                self.dataBox.isHidden = false
                
                if(minChangeTrigger == maxChangeTrigger){
                    hideTriggerBox = true
                }
                else{
                    self.minChangeTriggerLabel.stringValue = String(format:"%.2f", minChangeTrigger)
                    self.maxChangeTriggerLabel.stringValue = String(format:"%.2f", maxChangeTrigger)
                    self.changeTriggerLabel.stringValue = String(format:"%.2f ℃", changeTrigger)
                    self.changeTriggerSlider.doubleValue = changeTrigger
                    
                    self.changeTriggerSlider.minValue = minChangeTrigger
                    self.changeTriggerSlider.maxValue = maxChangeTrigger
                    self.changeTriggerSlider.doubleValue = changeTrigger
                }
                
                self.minDataIntervalLabel.stringValue = String(format:"%d",minDataInterval)
                self.maxDataIntervalLabel.stringValue = String(format:"%d",maxDataInterval)
                self.dataIntervalLabel.stringValue = String(format:"%d",dataInterval)
                
                self.dataIntervalSlider.minValue = Double(minDataInterval)
                self.dataIntervalSlider.maxValue = Double(maxDataInterval)
                self.dataIntervalSlider.doubleValue = Double(dataInterval)
                
                if(subclass == ChannelSubclass.temperatureSensorThermocouple){
                    self.thermocoupleTypePopup.addItem(withTitle: "J")
                    self.thermocoupleTypePopup.addItem(withTitle: "K")
                    self.thermocoupleTypePopup.addItem(withTitle: "E")
                    self.thermocoupleTypePopup.addItem(withTitle: "T")
                    self.thermocoupleTypePopup.selectItem(at: (thermocoupleType == ThermocoupleType.J) ? 0 : (thermocoupleType == ThermocoupleType.K) ? 1 : (thermocoupleType == ThermocoupleType.E) ? 2 : 3)
                    self.updateView(dataIntervalIsHidden: false, triggerIsHidden: hideTriggerBox, thermocoupleIsHidden: false, rtdIsHidden: true)
                }
                else if(subclass == ChannelSubclass.temperatureSensorRTD){
                    self.rtdTypePopup.addItem(withTitle: "PT100 3850")
                    self.rtdTypePopup.addItem(withTitle: "PT1000 3850")
                    self.rtdTypePopup.addItem(withTitle: "PT100 3920")
                    self.rtdTypePopup.addItem(withTitle: "PT1000 3920")
                    self.rtdTypePopup.selectItem(at: (rtdType == RTDType.PT100_3850) ? 0 : (rtdType == RTDType.PT1000_3850) ? 1 : (rtdType == RTDType.PT100_3920) ? 2 : 3)
                    
                    self.rtdWireSetupPopup.addItem(withTitle: "2 Wire")
                    self.rtdWireSetupPopup.addItem(withTitle: "3 Wire")
                    self.rtdWireSetupPopup.addItem(withTitle: "4 Wire")
                    self.rtdWireSetupPopup.selectItem(at: (rtdWireSetup == RTDWireSetup.wires_2) ? 0 : (rtdWireSetup == RTDWireSetup.wires_3) ? 1 : 2)
                    self.updateView(dataIntervalIsHidden: false, triggerIsHidden: hideTriggerBox, thermocoupleIsHidden: true, rtdIsHidden: false)
                }
                else{
                    self.updateView(dataIntervalIsHidden: false, triggerIsHidden: hideTriggerBox, thermocoupleIsHidden: true, rtdIsHidden: true)
                }
            
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    func detach_handler(sender: Phidget){
        DispatchQueue.main.async {
            self.phidgetInfoBox.fillPhidgetInfo(device: nil)
            self.dataIntervalBox.isHidden = true
            self.changeTriggerBox.isHidden = true
            
            self.dataBox.isHidden = true
            
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.minSize.height
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        outputError(errorDescription: data.description, errorCode: data.code.rawValue)
    }
    
    func temperaturechange_handler(sender: TemperatureSensor, temperature: Double){
        DispatchQueue.main.async {
            self.temperatureLabel.stringValue = String(format:"%.2f ℃", temperature)
        }
    }
    
    //MARK: GUI Controls
    @IBAction func setChangeTrigger(_ sender: Any) {
        let slider = sender as! NSSlider
        do{
            try ch.setTemperatureChangeTrigger(slider.doubleValue)
            changeTriggerLabel.stringValue = String(format:"%.2f ℃", slider.doubleValue)
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setDataInterval(_ sender: Any) {
        let slider = sender as! NSSlider
        do{
            try ch.setDataInterval(UInt32(slider.intValue))
            dataIntervalLabel.stringValue = String(format:"%d ms", UInt32(slider.intValue))
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setThermocoupleType(_ sender: Any) {
        do{
            let popup = sender as! NSPopUpButton
            switch(popup.indexOfSelectedItem){
            case 0:
                try ch.setThermocoupleType(ThermocoupleType.J)
                break
            case 1:
                try ch.setThermocoupleType(ThermocoupleType.K)
                break
            case 2:
                try ch.setThermocoupleType(ThermocoupleType.E)
                break
            case 3:
                try ch.setThermocoupleType(ThermocoupleType.T)
                break
            default:
                break
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    @IBAction func setRTDType(_ sender: Any) {
        do{
            let popup = sender as! NSPopUpButton
            switch(popup.indexOfSelectedItem){
            case 0:
                try ch.setRTDType(RTDType.PT100_3850)
                break
            case 1:
                try ch.setRTDType(RTDType.PT1000_3850)
                break
            case 2:
                try ch.setRTDType(RTDType.PT100_3920)
                break
            case 3:
                try ch.setRTDType(RTDType.PT1000_3920)
                break
            default:
                break
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setRTDWireSetup(_ sender: Any) {
        do{
            let popup = sender as! NSPopUpButton
            switch(popup.indexOfSelectedItem){
            case 0:
                try ch.setRTDWireSetup(RTDWireSetup.wires_2)
                break
            case 1:
                try ch.setRTDWireSetup(RTDWireSetup.wires_3)
                break
            case 2:
                try ch.setRTDWireSetup(RTDWireSetup.wires_4)
                break
            default:
                break
            }
        } catch let err as PhidgetError {
            outputError(errorDescription: err.description, errorCode: err.errorCode.rawValue)
        } catch {
            print (error)
        }
    }
    
    //MARK: Error
    func outputError(errorDescription:String, errorCode:UInt32){
        DispatchQueue.main.async {
            self.errorCounter += 1
            let outputString = NSAttributedString(string: String(format:"Error %d: %@\n", errorCode, errorDescription))
            self.errorEventLog.textStorage?.beginEditing()
            self.errorEventLog.textStorage?.append(outputString)
            self.errorEventLog.textStorage?.endEditing()
            self.errorEventCount.stringValue = "\(self.errorCounter)"
            if(!self.errorEventWindow.isVisible){
                self.errorEventWindow.setIsVisible(true)
                self.errorEventWindow.center()
            }
        }
        
    }
    
    @IBAction func clearError(_ sender: Any) {
        errorEventLog.textStorage?.setAttributedString(NSAttributedString(string: ""))
        errorCounter = 0
        errorEventCount.stringValue = "0"
    }
    
    //MARK: Misc
    func updateView(dataIntervalIsHidden:Bool, triggerIsHidden:Bool, thermocoupleIsHidden:Bool, rtdIsHidden:Bool){
        DispatchQueue.main.async {
            self.dataIntervalBox.isHidden = dataIntervalIsHidden
            self.changeTriggerBox.isHidden = triggerIsHidden
            self.thermocoupleBox.isHidden = thermocoupleIsHidden
            self.rtdBox.isHidden = rtdIsHidden
            
            //Adjusting window height
            var frame = self.window.frame
            frame.size.height = self.window.maxSize.height
            if(!thermocoupleIsHidden){
                frame.size.height += self.thermocoupleBox.frame.height
            }
            else if(!rtdIsHidden){
                frame.size.height += self.rtdBox.frame.height
            }
            self.window.setFrame(frame, display: true, animate: false)
            self.window.center()
        }
    }
}

